@@ -322,7 +322,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     QString                        myFolderName = QStringLiteral("ksane");
     QMap<QString, QString>         wallet_entry;
 
-    if (d->m_saneHandle != 0) {
+    if (d->m_saneHandle != nullptr) {
         // this KSaneWidget already has an open device
         return false;
     }
@@ -415,7 +415,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
     // Read the options (start with option 0 the number of parameters)
     optDesc = sane_get_option_descriptor(d->m_saneHandle, 0);
-    if (optDesc == 0) {
+    if (optDesc == nullptr) {
         d->m_auth->clearDeviceAuth(d->m_devName);
         d->m_devName.clear();
         return false;
@@ -529,7 +529,7 @@ bool KSaneWidget::closeDevice()
     d->m_auth->clearDeviceAuth(d->m_devName);
     // else
     sane_close(d->m_saneHandle);
-    d->m_saneHandle = 0;
+    d->m_saneHandle = nullptr;
     d->clearDeviceOptions();
 
     // disable the interface until a new device is opened.
@@ -547,6 +547,16 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
                                    int height,
                                    int bytes_per_line,
                                    ImageFormat format)
+{
+    return KSaneWidget::toQImageSilent(data, width, height, bytes_per_line, currentDPI(), format);
+}
+
+QImage KSaneWidget::toQImageSilent(const QByteArray &data,
+                                   int width,
+                                   int height,
+                                   int bytes_per_line,
+                                   int dpi,
+                                   ImageFormat format)
 {
     QImage img;
     int j = 0;
@@ -619,7 +629,7 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
         qDebug() << "Unsupported conversion";
         break;
     }
-    float dpm = currentDPI() * (1000.0 / 25.4);
+    float dpm = dpi * (1000.0 / 25.4);
     img.setDotsPerMeterX(dpm);
     img.setDotsPerMeterY(dpm);
     return img;
@@ -649,6 +659,16 @@ void KSaneWidget::scanFinal()
     }
 }
 
+void KSaneWidget::startPreviewScan()
+{
+    if (d->m_btnFrame->isEnabled()) {
+        d->startPreviewScan();
+    } else {
+        // if the button frame is disabled, there is no open device to scan from
+        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+    }
+}
+
 void KSaneWidget::scanCancel()
 {
     if (d->m_scanThread->isRunning()) {
@@ -685,7 +705,7 @@ bool KSaneWidget::getOptVal(const QString &optname, QString &value)
 {
     KSaneOption *option;
 
-    if ((option = d->getOption(optname)) != 0) {
+    if ((option = d->getOption(optname)) != nullptr) {
         return option->getValue(value);
     }
     // Special handling for non sane option
@@ -698,6 +718,11 @@ bool KSaneWidget::getOptVal(const QString &optname, QString &value)
 
 int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
 {
+    if (d->m_scanThread->isRunning() ||
+            d->m_previewThread->isRunning()) {
+        return -1;
+    }
+
     QString tmp;
     int i;
     int ret = 0;
@@ -746,9 +771,14 @@ int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
 
 bool KSaneWidget::setOptVal(const QString &option, const QString &value)
 {
+    if (d->m_scanThread->isRunning() ||
+            d->m_previewThread->isRunning()) {
+        return false;
+    }
+
     KSaneOption *opt;
 
-    if ((opt = d->getOption(option)) != 0) {
+    if ((opt = d->getOption(option)) != nullptr) {
         if (opt->setValue(value)) {
             if ((d->m_splitGamChB) &&
                     (d->m_optGamR) &&
@@ -793,7 +823,7 @@ bool KSaneWidget::setOptVal(const QString &option, const QString &value)
 
 void KSaneWidget::setScanButtonText(const QString &scanLabel)
 {
-    if (d->m_scanBtn == 0) {
+    if (d->m_scanBtn == nullptr) {
         qCritical() << "setScanButtonText was called before KSaneWidget was initialized";
         return;
     }
@@ -802,7 +832,7 @@ void KSaneWidget::setScanButtonText(const QString &scanLabel)
 
 void KSaneWidget::setPreviewButtonText(const QString &previewLabel)
 {
-    if (d->m_scanBtn == 0) {
+    if (d->m_scanBtn == nullptr) {
         qCritical() << "setPreviewButtonText was called before KSaneWidget was initialized";
         return;
     }
